#
# Copyright 2007 Fedora Unity
#
# Jonathan Steffan <jon a fedoraunity.org>
# Jeroen van Meeuwen <kanarip a fedoraunity.org>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; version 2 only
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
#

import os
import pykickstart
import sys

import pykickstart
import pykickstart.constants as constants
import pykickstart.parser as parser
from urlgrabber import urlgrab

# Translation
from rhpl.translate import _

class RevisorKickstart:
    """A robust kickstart object interfacing between Revisor and different pykickstart APIs."""
    def __init__(self, cfg):
        self.cfg = cfg

    def _reset(self):
        self.parser._reset()

    def create_parser(self):
        """Returns a regular kickstart parser"""
        try:
            import pykickstart.version
            self.handler = pykickstart.version.makeVersion(version=pykickstart.version.stringToVersion(self.cfg.version_from))
            self.parser = pykickstart.parser.KickstartParser(self.handler, missingIncludeIsFatal=False)
        except:
            import pykickstart.data
            self.ksdata = pykickstart.data.KickstartData()
            self.handler = pykickstart.parser.KickstartHandlers(self.ksdata)
            self.parser = pykickstart.parser.KickstartParser(self.ksdata, self.handler, missingIncludeIsFatal=False)

    def _handler(self):
        """Returns the kickstart data object instance if it's there,
        and if it's not, it returns the handler"""
        if hasattr(self,"ksdata"):
            return self.ksdata
        else:
            return self.handler

    def read_file(self, url):
        """Reads in a kickstart file and returns True on success or False on failure"""
        # FIXME
        # For some reason the code below shows 0 byte sized kickstarts at the new location in /var/tmp
        # urlgrabber though has urlgrab() and urlopen(), the latter of which is used by pykickstarts
        # readKickstart()
        #
        #kickstart_file_name = os.path.join(self.cfg.working_directory, "%s-ks.cfg" % self.cfg.model)
        #try:
            #urlgrab(url, filename=kickstart_file_name, copy_local=1)
        #except Exception, e:
            #self.cfg.log.error(_("Download of %s to %s failed." % (url, kickstart_file_name)), recoverable=False)
        #self.cfg.kickstart_file = kickstart_file_name
        #print "Kickstart file: %s" % self.cfg.kickstart_file
        #self.parser.readKickstart(kickstart_file_name)
        self.parser.readKickstart(url)

    def _reset(self):
        self.parser._reset()

    def __str__(self):
        if hasattr(self,"ksdata"):
            kswriter = pykickstart.writer.KickstartWriter(self.ksdata)
            return kswriter.write()
        else:
            return "%s" % self.handler.__str__()

    def _get(self, item=None, val=None):
        # pykickstart >= 0.100
        if not item == None:
            if hasattr(self,"ksdata"):
                self_ref = self.ksdata
            else:
                self_ref = self.handler

            if hasattr(self_ref,item):
                if not val == None:
                    if hasattr(getattr(self_ref,item),val):
                        return getattr(getattr(self_ref,item),val)
                    elif isinstance(getattr(self_ref,item), dict):
                        return getattr(self_ref,item)[val]
                    else:
                        pass
                else:
                    return getattr(self_ref,item)
            elif hasattr(self_ref,val):
                return getattr(self_ref,val)

        elif hasattr(self,"ksdata"):
            return self.ksdata
        else:
            return self.handler

    def _set(self, item1, item2=None, val=None):
        # ks.item1 = val
        # ks.item1.item2 = val
        # ks.item1[item2] = val

        if hasattr(self,"ksdata"):
            self_ref = self.ksdata
        else:
            self_ref = self.handler

        if hasattr(self_ref,item1):
            if not item2 == None:
                self_ref = getattr(self_ref,item1)
                if hasattr(self_ref,item2):
                    setattr(self_ref,item2,val)
                elif isinstance(getattr(self_ref,item2), dict):
                    self_ref[item2] = val
                else:
                    raise AttributeError
            else:
                setattr(self_ref,item1,val)
        else:
            raise AttributeError

    def _NetworkData(self):
        if hasattr(self,"ksdata"):
            self.ks_net_data = pykickstart.data.KickstartNetworkData()
        else:
            self.ks_net_data = self.handler.NetworkData()
        return self.ks_net_data

    def _network_add(self, nd):
        if hasattr(self,"ksdata"):
            self.ksdata.network.add(nd)
        else:
            self.handler.network.add(nd)

    def _Group(self, name, include=None):
        if hasattr(self,"ksdata"):
            group = pykickstart.data.packages.Group(name)
        else:
            if include == None:
                include = constants.GROUP_DEFAULT
            group = pykickstart.parser.Group(name, include=include)
        return group
